# store row/element subscripts for undisturbed age classes 
dist_ages <- seq(1, 55, 6)
# matrix to store values that will be retained from all simulations, now extended to 5 values from original 4
# age.out stores full age structure for disturbed and undisturbed populations at start of breeding season age.out[,1&2,]

dat.out <- array(NA, dim = c((years+1), 5, nboot))
age.out <- array(NA, dim = c((years+1), 2, 10, nboot))

# NdistSum is the total population size for the disturbed population each year, NNotDistSum is the equivalent for the matching undisturbed population
colnames(dat.out) <- c('NdistSum', 'NnotdistSum','Nnotdist-Ndist', '% decline Ndist', '% decline Nnotdist')

# matrix to store total number of days of disturbance experienced by dependents, juveniles and adults 
# in the moderate disturbance class (d_days.out[,,1,]), 
# and the number of individuals who experience this much disturbance (d_days.out[,,2,])
d_days.out <- array(c(0), dim = c(pile_years, 3, 2, nboot))
# matrix to store number of animals experiencing PTS in each year of piling
PTS_record <- array(c(0), dim = c(pile_years,nboot))

# skip everything from here to line 38 if there is no piling
if (pile_years > 0){
  
# p_disturb_base and p_pts_base contain baseline probabilities for disturbance and PTS for each vulnerable sub-pop
# on each day. They are multiplied by the value of sdNDt for each bootstrap to get the values for each set of simulations
p_disturb_base  <- matrix(0, nrow = pile_years * 365, ncol = nvulnmean)
colnames(p_disturb_base) <- paste("sub-pop", 1:nvulnmean, sep = '')
p_pts_base <- p_disturb_base

for (j in 1:(nvulnmean)) { 
  for(k in 1:nrow(p_disturb_base))  {
    p_disturb_base[k, j] <- ifelse( sum((pile[k, pilesx] * vulnpile[j, pilesx] * NDt[k, pilesx]) / (vulnmean[j] * pmean)) > 1.0, 1.0, 
                                    sum((pile[k, pilesx] * vulnpile[j, pilesx] * NDt[k, pilesx]) / (vulnmean[j] * pmean))) 
    
    p_pts_base[k, j]     <- sum(pile[k, pilesx] * vulnpile[j, pilesx] * NPt[k, pilesx]) / sum(pile[k, pilesx] * vulnpile[j, pilesx] * NDt[k, pilesx])
  }
}

p_pts_base[is.na(p_pts_base)] <- 0  
}

# matrices to store numbers affected in different ways in a more accessible format than Ndist for last iteration of bootstrap.   
Ndist_out_1 <- Ndist_out_2 <- array(0, dim = c(10, 7, pile_years))
colnames(Ndist_out_1) <- colnames(Ndist_out_2) <- c('age', 'not disturbed','mod dist', 'severe dist', 'PTS only','PTS+mod dist','PTS+severe dist')

for(boot_number in 1:nboot) { 
#boot_number <- 1
  
  Ndist <- matrix(0, nrow = 60, ncol = years+1)  
#  Nnotdist <- matrix(0, nrow = 10, ncol = years+1)
  NColl_byage <- rep(0,60)
  # Nnotdist_allvuln is second matrix to check what happens if vulnerable sub-populations are specified
  Nnotdist_allvuln <- matrix(0, nrow = 60, ncol = years+1)  
  
  # set up identical age structures for disturbed and undisturbed components of population  
  Ndist[dist_ages, 1] <- Nnotdist_allvuln[dist_ages,1] <- round(pmean * age_structure[1:10], 0)
  dat.out[1,1, boot_number] <- round(sum(Ndist[1:60,1])/propfemale, 0)
  for(isum in 1:10){
    age.out[1,1,isum,boot_number] <- round(sum(Ndist[dist_ages[isum]:(dist_ages[isum]+5),1])/propfemale,0)
    age.out[1,2,isum,boot_number] <- round(sum(Nnotdist_allvuln[dist_ages[isum]:(dist_ages[isum]+5),1])/propfemale,0)}
  dat.out[1,2, boot_number] <- round(sum(Nnotdist_allvuln[1:60,1])/propfemale, 0)
  dat.out[1,3,boot_number] <- dat.out[1,1,boot_number] - dat.out[1,2,boot_number]
  # vector to hold fecundity values for current year
  AnnFert<-rep(c(0),6)

    # call f.experts.survival&fertility to sample bootstrap values from the expert elicitation
  tmp  <- f.experts.survival_and_fertility(spec) 
  Surv <- tmp$Surv
  Fert <- tmp$Fert

    # Number of days of disturbance required for moderate and severe disturbance of dependent (pups/calves) (dd1), juveniles (dd2) and adults (dd3)
  dd1  <- tmp$dd1 
  dd2  <- tmp$dd2
  dd3  <- tmp$dd3
  
  # These vectors contain the relationships between pup survival, juvenile survival and adult fecundity and number of day of disturbance
  depend_days <- tmp$depend_days
  juvenile_days <- tmp$juvenile_days
  adult_days <- tmp$ adult_days
  
  # create matrix with dd's for each age class, to be used in function f.disturbance.classes.2
  dd_vec <- c(rep(c(dd1),c(age1)),rep(c(dd2),c(age2-age1)),rep(c(dd3),c(10-age2)))
  dds <- matrix(dd_vec,nrow=10,ncol=3,byrow=TRUE)
  
  # incorporate uncertainty into NDt, NPt and NCollisions, 95% confidence limits are mean +/- 50%
  sdNDt <- rep(0,nvulnmean)
  sd <- exp(rnorm(1, 0, 0.25))
#  sd <- 1
  sdNDt[1:nvulnmean]  <- sd
  NColl_var <- round(NCollisions*sd)
  
  # d_days will contain mean number of days of disturbance experienced by animals with and without PTS (d_days[,2] and d_days[,1])
  # d_days[1,] is for dependent (calves or pups), d_days[2,] is for juveniles, d_days[3,] is for adults
  d_days <- array(c(0), dim = c(3,2))
  
  # skip everything from here to line 266 if there is no piling
  if (pile_years > 0){
 
    # create array to hold age structures of each vulnerable sub-population and undisturbed remainder  
    Ndistvuln <- array(0, dim = c(60, length(newvulnmean),(pile_years + 1)))
    # create a similar array for the undisturbed population
    Nnotdistvuln <- array(0, dim = c(60, length(newvulnmean),(pile_years + 1)))
    # determine age structures for each vulnerable sub-population, so that their combined age structure is the same as Ndist
    for (ivuln in dist_ages){
      Ndistvuln[ivuln,1:length(newvulnmean),1] <- rmultinom(1,Ndist[ivuln,1],newvulnmean)
    }
    Nnotdistvuln[,,1] <- Ndistvuln[,,1]
    if(dim(Ndistvuln)[2]== 1){Nnotdist_allvuln[,1] <- Nnotdistvuln[,1,1]} else {Nnotdist_allvuln[,1] <- rowSums(Nnotdistvuln[,,1])}
    
   
    p_disturb <- p_disturb_base*sdNDt
    p_pts <- p_pts_base*sdNDt
    p_disturb[p_disturb > 1] <- 1
    p_pts[p_pts > 1] <- 1
    
#     dist_record <- 0
#    Ndist_record <- matrix(c(0), nrow = 60, ncol = pile_years)
           
    for(t in 1:pile_years){       
      
      # create quantiles for AnnFert and Surv to use in demographic stochasticity calculations
      # the same quantile will be used for all Surv values that apply to a particular age class
      F_quantile <- runif(1,0,1)
      S_quantile <- rep(c(0), 60)
      # choose quantiles for each age-class
      S_quantile[dist_ages] <- runif(10,0,1)
      # distribute quantiles across all other appropriate elements of S_quantile
      for (i_quant in 1:5){S_quantile[dist_ages+i_quant] <- S_quantile[dist_ages]}
      
      #  recalculate vuln so that it matches with rescaled sub-population age structures 
      if(dim(Ndistvuln)[2] == 1){vuln <-sum(Ndistvuln[,,t])}else{vuln <-colSums(Ndistvuln[,,t])}
      wvec <- which(vuln[1:nvulnmean] > 1000)
      vuln[wvec] <- 1000   

      # determine fecundity rates for current year 
      Fert_undist <- propfemale*(rbeta(1, a[3], b[3]))
      #Fert_undist <- Fertility*propfemale
      if (z == 0){AnnFert[1] <- Fert_undist} else {
        #calculate density dependent multiplier for undisturbed population
        Nnotdist_to_K <- (sum(Nnotdist_allvuln[,t]))/K
        ddFert_undist <- Fert_K + (Fert_0 - Fert_K)*(1 - Nnotdist_to_K^z)
        if (ddFert_undist < 0) ddFert_undist <- 0
        
        #calculate density dependent multiplier for disturbed population
        Ndist_to_K <- (sum(Ndist[,t]))/K
        ddFert_dist <- Fert_K + (Fert_0 - Fert_K)*(1- Ndist_to_K^z)
        if (ddFert_dist < 0) ddFert_dist <- 0
        
        # Account for environmental stochasticity
        AC_environment <- Fert_undist/Fertility
        Fert_undist <- ddFert_undist*AC_environment
        AnnFert[1] <- ddFert_dist*AC_environment
      }
      
      #    Fert_record[t,1, boot_number] <- Fert_undist
      #    Fert_record[t,2, boot_number] <- AnnFert[1]
      
      AnnFert[2:6] <- Fert[2:6]*AnnFert[1] 
      
      #determine pup and juvenile survival values for current year, accounting for environmental stochasticity
       calfsurv    <- rbeta(1,a[1],b[1])
#       calfsurv    <- Surv[1]
       juvsurv     <- rbeta(1,a[2],b[2])
#       juvsurv     <- Surv[7]
      
     for(i in 1:length(newvulnmean)){ # start the loop over the multiple vulnerable sub-pops; 
        yrs <- which(yearvec == t)
        if (vuln[i]>0){
          if (i<=nvulnmean){
          tmp       <- f.disturbance_and_PTS(pile[yrs, max(pilesx) + 2  + i], vuln[i], days, other_days, p_disturb[yrs, i], p_pts[yrs, i], prop_days_dist, Avoid)
          dist_year <- tmp$dist_year
          pts_year  <- tmp$pts_year  
        dist_year[,366] <-  apply(dist_year,1,sum)
        dist_year[,367] <-  ifelse(apply(pts_year,1,sum) > 0, 1, 0)
        # record total number of animals experiencing PTS in year t
        PTS_record[t,boot_number] <- sum(dist_year[,367])/propfemale
#        dist_record[t] <- mean(dist_year[,366])
#        Ndist_record[,t] <- Ndistvuln[,i,t]
        # assign individuals in sub-population to different disturbance classes
        tmp       <- f.disturbance.classes.2(dist_year, pts_year, Ndistvuln[,i,t], dist_ages, days, other_days, dds)
        Ndistvuln[,i,t]    <- tmp$N  
        dist_days <-tmp$dist_days

               
        # create a temporary array that will redistribute individuals in Ndistvuln into a more manageable form
        Ntemp3 <- Ndistvuln[1:60,i,t]
        Ntemp4 <- matrix(Ntemp3, nrow = 10, ncol = 6, byrow = TRUE)

        # determine mean number of days of disturbance for dependents (calves or pups).
        d_days[1,1] <- round(sum(dist_days[1:age1,1])/sum(Ntemp4[1:age1,2]),0)
        d_days[1,2] <- round(sum(dist_days[1:age1,2])/sum(Ntemp4[1:age1,5]),0)
        
        # determine mean number of days of disturbance for juveniles.Subscript chosen to match code for disturb_postX$
        d_days[2,1] <- round((sum(dist_days[(age1+1):age2,1])/sum(Ntemp4[(age1+1):age2,2])),0)
        d_days[2,2] <- round((sum(dist_days[(age1+1):age2,2])/sum(Ntemp4[(age1+1):age2,5])),0)
        
        # determine mean number of days of disturbance for adults. Subscript chosen to match code for disturb_postX$
        d_days[3,1] <- round((sum(dist_days[(age2+1):10,1])/sum(Ntemp4[(age2+1):10,2])),0)
        d_days[3,2] <- round((sum(dist_days[(age2+1):10,2])/sum(Ntemp4[(age2+1):10,5])),0)
        
        # CHECK FOR CASES WHERE d_days is NaN using is.nan
        for (jNaN in 1:3){
          for (kNaN in 1:2){
            if (is.nan(d_days[jNaN,kNaN])) {d_days[jNaN,kNaN] <- 0}
#            if (d_days[jNaN,kNaN] < 1) {d_days[jNaN,kNaN] <- 1}
                          }
                        }
        # store mean number of days of disturbance experienced by moderately disturbed animals with PTS d_days.out[t,,2,]
        # and mean number of days of disturbance experienced by moderately disturbed animals without PTS d_days.out[t,,1,] 
        # for dependents d_days.out[t,1,], juveniles d_days.out[t,2,]and adults d_days.out[t,3,]
        d_days.out[t,1,1,boot_number] <- d_days[1,1]
        d_days.out[t,2,1,boot_number] <- d_days[2,1]
        d_days.out[t,3,1,boot_number] <- d_days[3,1]
        d_days.out[t,1,2,boot_number] <- d_days[1,2]
        d_days.out[t,2,2,boot_number] <- d_days[2,2]
        d_days.out[t,3,2,boot_number] <- d_days[3,2]
         }

        # call f.current.survival.values.3 to determine survival values for each disturbance category in current year
        AnnSurv   <- f.current.survival.values.3(dist_ages, age1, age2, calfsurv, juvsurv, d_days, juvenile_days, depend_days)
        Fert[2] <- adult_days[d_days[3,1]+1]
        Fert[5] <- Fert[4] * adult_days[d_days[3,2]+1]
        AnnFert[2:6] <- Fert[2:6]*AnnFert[1]
        
        # call f.next.year to determine numbers in each age-class of the vulnerable sub-populations in next year
        tmp                <- f.next.year(Ndistvuln[,i,t], Ndistvuln[,i,t+1], dist_ages, pmean, AnnFert, threshold, age1, age2, F_quantile, S_quantile)
        Ndistvuln[,i,t+1]  <- tmp$N2
        tmp               <- f.next.year(Nnotdistvuln[,i,t], Nnotdistvuln[,i,t+1], dist_ages, pmean, AnnFert, threshold, age1, age2, F_quantile, S_quantile)
        Nnotdistvuln[,i,t+1]  <- tmp$N2
        }
        
      } 
      # end i loop over vulnerable sub-populations  
      
      # combine sizes of all disturbed sub-populations before any mortality is applied and store values 
      if(dim(Ndistvuln)[2]== 1){Ntemp <- Ndistvuln[1:60,,t]} else {Ntemp <- rowSums(Ndistvuln[1:60,,t])}
      
      Ndist_out_1[,2:7,t] <- matrix(Ntemp, nrow = 10, ncol = 6, byrow = TRUE)
 
            # combine all vulnerable sub-populations into Ndist   
      if(dim(Ndistvuln)[2]== 1){Ntemp2 <- Ndistvuln[1:60,,t+1]} else {Ntemp2 <- rowSums(Ndistvuln[1:60,,t+1])}
      Ndist[,t+1] <- Ntemp2
      Ndist_out_2[,2:7, t] <- matrix(Ntemp2, nrow = 10, ncol = 6, byrow = TRUE)
      # do the same for the undisturbed populations
      if(dim(Ndistvuln)[2]== 1){Ntemp3 <- Nnotdistvuln[1:60,,t+1]} else {Ntemp3 <- rowSums(Nnotdistvuln[1:60,,t+1])}
      Nnotdist_allvuln[,t+1] <- Ntemp3
      
      # distribute deaths due to collisions across age classes
      if (NCollisions != 0 & sum(Ndist[,t+1])>0) {
        # calculate probability of death for each age class
        P_collision <- Ndist[,t+1]/sum(Ndist[,t+1])
        NColl_byage <-   rmultinom(1, NColl_var, P_collision)
        # account for mortality from other causes
        NColl_byage <- sapply(1:60, function(X) rbinom(1,NColl_byage[X],sqrt(AnnSurv[X])))
        #remove deaths by collision
        Ndist[,t+1] <- Ndist[,t+1] - NColl_byage
        # check for situations where more individuals are killed than exist                                                                                                                                                                     
        overkills <- which(Ndist[,t+1]<0)
        Ndist[overkills,t+1] <- 0
      } 
      
      t2 <- t+1
      # save age structure
      for(isum in 1:10){
        age.out[t2,1,isum,boot_number] <- round(sum(Ndist[dist_ages[isum]:(dist_ages[isum]+5),t2])/propfemale,0)}
        age.out[t2,2,,boot_number] <- round(Nnotdist_allvuln[dist_ages,t2]/propfemale,0)
      
      Ndist_out_1[,1,t] <- Ndist_out_2[,1,t] <- c(0:9)
    }
# end loop over piling years

  }
  # start of calculations if pile_years = 0   
#################################################################
  
  
  
for(t in (pile_years + 1):(years)){
  # if there is no effect of PTS on adult survival, move all adults with PTS into the unaffected age class  
  if (Surv[16]==1){
    Ndist[dist_ages[age2:10],t] <- Ndist[dist_ages[age2:10],t] + Ndist[(dist_ages[age2:10]+3),t]
    Ndist[(dist_ages[age2:10]+3),t] <- 0}
  # create quantiles for AnnFert and Surv to use in demographic stochasticity calculations
  # the same quantile will be used for all Surv values that apply to a particular age class
  F_quantile <- runif(1,0,1)
  S_quantile <- rep(c(0), 60)
  # choose quantiles for each age-class
  S_quantile[dist_ages] <- runif(10,0,1)
  # distribute quantiles across all other appropriate elements of S_quantile
  for (i_quant in 1:6){S_quantile[dist_ages+i_quant] <- S_quantile[dist_ages]}
  
    # determine fecundity rates for current year 
    #Fert_undist <- propfemale*(rbeta(1, a[3], b[3]))
    Fert_undist <- propfemale*Fertility
    if (z == 0){AnnFert[1] <- Fert_undist} else {
      #calculate density dependent multiplier for undisturbed population
      Nnotdist_to_K <- (sum(Nnotdist_allvuln[,t]))/K
      ddFert_undist <- Fert_K + (Fert_0 - Fert_K)*(1 - Nnotdist_to_K^z)
      if (ddFert_undist < 0) ddFert_undist <- 0

      #calculate density dependent multiplier for disturbed population
      Ndist_to_K <- (sum(Ndist[,t]))/K
      ddFert_dist <- Fert_K + (Fert_0 - Fert_K)*(1- Ndist_to_K^z)
      if (ddFert_dist < 0) ddFert_dist <- 0
    
      # Account for environmental stochasticity
      AC_environment <- Fert_undist/Fertility
      Fert_undist <- ddFert_undist*AC_environment
      AnnFert[1] <- ddFert_dist*AC_environment
    }
    
#    Fert_record[t,1, boot_number] <- Fert_undist
#    Fert_record[t,2, boot_number] <- AnnFert[1]
    
    AnnFert[2:6] <- Fert[2:6]*AnnFert[1] 
    
    #determine survival values for current year, accounting for environmental stochasticity
#    calfsurv    <- rbeta(1,a[1],b[1])
#    juvsurv     <- rbeta(1,a[2],b[2])
    calfsurv    <- Surv[1]
    juvsurv     <- Surv[7]
    
    # call f.current.survival.values.3 to determine survival values for each disturbance category in current year
    AnnSurv   <- f.current.survival.values.3(dist_ages, age1, age2, calfsurv, juvsurv, d_days, juvenile_days, depend_days)

    # call next.year to determine numbers in each age-class in next year                                                                                                                 
    tmp         <- f.next.year(Ndist[, t], Ndist[, t + 1], dist_ages, pmean, AnnFert, threshold, age1, age2, F_quantile, S_quantile) 
    Ndist[,t+1] <- tmp$N2
    tmp         <- f.next.year(Nnotdist_allvuln[,t], Nnotdist_allvuln[, t + 1], dist_ages, pmean, AnnFert, threshold, age1, age2, F_quantile, S_quantile) 
    Nnotdist_allvuln[,t+1] <- tmp$N2
    
    
    # distribute deaths due to collisions across age classes
    if (NCollisions != 0 & sum(Ndist[,t+1])>0) {
      # calculate probability of death for each age class
      P_collision <- Ndist[,t+1]/sum(Ndist[,t+1])
      NColl_byage <-   rmultinom(1, NColl_var, P_collision)
      # account for mortality from other causes
      NColl_byage <- sapply(1:60, function(X) rbinom(1,NColl_byage[X],sqrt(AnnSurv[X])))
      #remove deaths by collision
      Ndist[,t+1] <- Ndist[,t+1] - NColl_byage
      # check for situations where more individuals are killed than exist                                                                                                                                                                     
      overkills <- which(Ndist[,t+1]<0)
      Ndist[overkills,t+1] <- 0
      } 
    
    t2 <- t+1
    for(isum in 1:10){
      age.out[t2,1,isum,boot_number] <- round(sum(Ndist[dist_ages[isum]:(dist_ages[isum]+5),t2])/propfemale,0)}
      age.out[t2,2,,boot_number] <- round(Nnotdist_allvuln[dist_ages,t2]/propfemale,0)

  }# end loop over post-Piling years
 
  # save output. dat.out[,1,] is the total size of the disturbed population
  # dat.out[,2,] is the size of the undisturbed population, dat.out[,3,] is the difference between the two populations
  # dat.out[,4,] is the annual change in the size of the disturbed population as a percentage
  # dat.out[,5,] is the annual change in the size of the undisturbed population as a p900ercentage
  dat.out[,1, boot_number] <- round(colSums(Ndist)/propfemale, 0)
  dat.out[,2, boot_number] <- round(colSums(Nnotdist_allvuln)/propfemale, 0)
  dat.out[,3,boot_number] <- dat.out[,1,boot_number] - dat.out[,2,boot_number]
  t_index <- c(2:t2)
  dat.out[t_index,4,boot_number] <- 100*((dat.out[t_index,1,boot_number]/(pmean/propfemale))^(1/(t_index-1))-1)
  dat.out[t_index,5,boot_number] <- 100*((dat.out[t_index,2,boot_number]/(pmean/propfemale))^(1/(t_index-1))-1)  
  
}
 # end the bootstrap


# save number of animals in each disturbance class, before and after mortality is applied, for the years piling occurs for the last bootstrap
# save modifications to survival and fecundity associated with PTS and disturbance for last bootstrap
# save summary statistics

save(pmean,vulnmean,pile_years,years,threshold,days,other_days,prop_days_dist,nboot,Surv,Fert,age_structure,dat.out,age.out,d_days.out,file = paste(spec, run_ID,'_Output.rdata', sep = ''))


